<?php

declare(strict_types=1);

namespace Drupal\vendor_stream_wrapper\Service;

use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\vendor_stream_wrapper\Event\VendorStreamWrapperCollectSafeListRegexPatternsEvent;
use Drupal\vendor_stream_wrapper\Event\VendorStreamWrapperEvents;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Service providing management functionality for vendor stream wrappers.
 */
class VendorStreamWrapperManager implements VendorStreamWrapperManagerInterface {

  /**
   * List of safe-list patterns.
   *
   * @var string[]|null
   */
  protected $patterns;

  /**
   * Creates a new VendorStreamWrapperManager instance.
   *
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $streamWrapperManager
   *   The Stream Wrapper Service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   A request stack object.
   * @param \Symfony\Contracts\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher.
   */
  public function __construct(
    protected StreamWrapperManagerInterface $streamWrapperManager,
    protected RequestStack $requestStack,
    protected EventDispatcherInterface $eventDispatcher,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getSafeListRegexPatterns(): array {
    if ($this->patterns === NULL) {
      // Collect the safe-list patterns by dispatching the appropriate event.
      $this->eventDispatcher->dispatch(
        new VendorStreamWrapperCollectSafeListRegexPatternsEvent($this),
        VendorStreamWrapperEvents::COLLECT_SAFE_LIST_REGEX_PATTERNS
      );
    }

    return $this->patterns;
  }

  /**
   * {@inheritdoc}
   */
  public function addSafeListRegexPatterns(array $patterns): void {
    $this->patterns = array_merge($this->patterns ?? [], $patterns);
  }

  /**
   * {@inheritdoc}
   */
  public function isSafeListed(string $file_path): bool {
    foreach ($this->getSafeListRegexPatterns() as $pattern) {
      // In case a valid pattern matches, indicate that the particular file is
      // allowed to be accessed.
      if (preg_match($pattern, $file_path)) {
        return TRUE;
      }
    }

    // No valid pattern matches the provided file path.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function createUrlFromUri(string $uri, bool $include_base_url = TRUE): ?string {
    if (str_starts_with($uri, 'vendor://')) {
      if ($wrapper = $this->streamWrapperManager->getViaUri($uri)) {
        $url = $wrapper->getExternalUrl();
        if ($include_base_url) {
          return $url;
        }
        $base_url = $this->requestStack->getCurrentRequest()?->getBaseUrl();
        return substr($url, strlen($base_url !== '/' ? $base_url : ''));
      }
    }
    else {
      return $uri;
    }

    return NULL;
  }

}
